
#ifndef BST_H
#define BST_H

#include "Edge.h"
#include <vector>
#include "ValHeap.h"
#include "CntxArray.h"
#include "ECString.h"
#include <iostream>
#include "SentRep.h"

//lhuang
#include <set>
#include <sstream>
#include <fstream>

#define NORMALVAL 0
#define TERMINALVAL 1
// lhuang: spurious
#define EXTRAVAL 2
class Bst;

// lhuang: list of tail nodes in a hyperedge
typedef list<Bst*> Bsts;
typedef vector<short> shorts;
typedef shorts::iterator shortIter;
typedef vector<Val*> BSType; // BS-star

typedef vector<Bst*> Cellstype[500][500];

// lhuang: Val is a hyperedge or derivation? i think it's a <e, vecj> pair
class Val
{
 public:
  static Val* newIth(int ith, Val* oval, bool& stop);
  Val() : status(NORMALVAL), len_(1), prob_(0), edge_(NULL), trm_(-1), wrd_(-1), visited(false)
    {
      vec_.push_back(0);
    }
  Val(Edge* e, double prb);
  ~Val();
  Edge* edge() const { return edge_; }
  Bsts& bsts() { return bsts_; }
  short  len() const { return len_; }
  short& len() { return len_; }
  short trm() const;
  short& trm1() { return trm_; }
  int wrd() const;
  int& wrd1() { return wrd_; }
  vector<short>& vec() { return vec_; }
  vector<short> vec() const { return vec_; }
  short&  vec(int i) { return vec_[i]; }
  double prob() const { return prob_; }
  double& prob() { return prob_; }
  // lhuang: figure of merit
  double fom() { return prob_; }
  void  extendTrees(Bst& bst2,int pos); 
  friend ostream& operator<<(ostream& os, const Val& v);
  friend bool operator==(Val& v1, Val& v2);
  bool check();
  short status;
  bool isTerminal();
  bool isSpurious() {
	return (!edge() && status == EXTRAVAL);
  }
  
  // lhuang: the label of the head node of this hyperedge; from extraMain.C : inputTreeFromBsts()
  ECString nodeLabel(SentRep &sr, int &pos);
  // returns " -> NP [2-3]  VP [3-5]
  ECString RHS();
  
  bool visited;
  double logprob;
 private:
  Val(Val* oval);
  short len_;
  double prob_;
  Edge*  edge_;
  short trm_;
  int   wrd_;
  Bsts bsts_;
  vector<short> vec_;
  // lhuang
  ECString label;
};

// lhuang: Bst is a node
class Bst
{
 public:
  Bst() : explored_(false), done_(false), num_(0), sum_(0) {}
  ~Bst();
  Val* next(int n);
  bool explored() const { return explored_; }
  bool& explored() { return explored_; }
  Val* nth(int i) { return nbest[i]; }
  int num() const { return num_; }
  int& num() { return num_; }
  bool empty() { return num_ == 0; }
  double prob() { return num_ == 0 ? 0 : nbest[0]->prob(); }
  double sum() const { return sum_; }
  double& sum() { return sum_; }
  ValHeap heap;
  // lhuang: now have a separate BS (hyperedges)
  void push(Val* val); 

  Val* pop() { return heap.pop(); }
  // lhuang
  int numHyperedges() { return BS.size(); } 
  void prepareNode(set<Bst*> & visited, SentRep & sr, int & pos, Cellstype & cells, int & count, int dep=0);
  ECString labelSpan(bool includeId=false);

  bool isTerminal() { assert (BS.size() > 0); return BS[0]->isTerminal(); }

  void addnth(Val* val)
    {
      num_++;
      nbest.push_back(val);
    }
  static void tester(Val* val);
  bool ptst(Val* val);
  BSType BS; // incoming hyperedges (Backward-Star)
  ECString label;
  int start, end;
  double logprob;
  int id;
 private:
  bool explored_;
  bool done_;
  int num_;
  double sum_;
  vector<Val*> nbest;
  //lhuang
};

typedef map<CntxArray, Bst, less<CntxArray> > BstMap;
Bst& bstFind(CntxArray& hi, BstMap& bm);
Bst&  ithBst(int i, Bsts& bsts);

void printForest(ofstream & fout, Bst & bst, SentRep & sr);

#endif /* ! BST_H */

